"""
FTP access for Aibo
"""

from ftplib import FTP
from os import path

WLANCONFOPTS = ["HOSTNAME", "ESSID", "WEPENABLE", "WEPKEY", "APMODE", "CHANNEL", "USE_DHCP", "IP_GATEWAY", "ETHER_NETMASK", "ETHER_IP"]

### misc / helper functions ###################################################

def validname(filename):
    dirname, filename = path.split(filename)
    name, ext = path.splitext(filename)
    if len(name) > 8 or len(ext) > 4:
        raise ValueError, "Invalid filename (longer than 8 characters)"

def checkfilename(func):
    """Decorator for checking if filename parameters are valid"""
    def newfunc(*args,**kwargs):
        validname(args[1])
        args = list(args)
        args[1] = args[1].upper()
        args = tuple(args)
        return func(*args, **kwargs)
    newfunc.__doc__ = func.__doc__
    newfunc.__name__ = func.__name__
    newfunc.__dict__.update(func.__dict__)
    return newfunc

### Aibo FTP class ############################################################

class AiboFTP:
    """Simplifies common file transfer tasks (upload/download objs, motions, sounds, etc.)"""
    def __init__(self, host):
        """Create ftp object and connect"""
        self.connect(host)

    def connect(self, host):
        """Manually connect to Aibo ftp server (e.g. after a disconnect)"""
        self._ftp = FTP(host)
        self._ftp.login("ftpinst", "ftpinst")
        print self._ftp.welcome

    def close(self):
        """Quit and close connection to Aibo ftp server"""
        self._ftp.quit()

    def cd(self, dir):
        """Change directory"""
        if dir == "/":
            self._ftp.cwd("/MS")
        else:
            self._ftp.cwd(dir)

    def pwd(self):
        """Print the current working directory"""
        return self._ftp.pwd()

    def ls(self, path=None):
        """List directory contents"""
        if dir:
            self._ftp.dir(path)
        else:
            self._ftp.dir()

    def dir(self, path):
        """More helpful directory listing"""
        ##if the file is a directory, show <DIR>
        ##otherwise, show <ext>
        ##
        ##size in bytes if less than 1kb (e.g. 924 bytes)
        ##size in kb if less than 1mb (e.g. 9.24 kb)
        ##size in mb otherwise (e.g. 9.2 mb)
        ##
        ##sort by filetype, then by filename, directories first
        ##
        ##remove timestamps
        ##
        ##allow wildcards
        pass

    @checkfilename
    def transferObj(self, binfile):
        """Upload object file (*.bin)"""
        #TODO: needs testing
        dirname, filename = path.split(binfile)
        self._ftp.storbinary("STOR /MS/OPEN-R/MW/OBJS/" + filename, file(binfile), 1024)

    @checkfilename
    def transferPos(self, posfile):
        """Upload posture file (*.pos)"""
        dirname, filename = path.split(posfile)
        self._ftp.storlines("STOR /MS/DATA/MOTION/" + filename, file(posfile))

    @checkfilename
    def transferMot(self, motfile):
        """Upload motion file (*.mot)"""
        dirname, filename = path.split(motfile)
        self._ftp.storlines("STOR /MS/DATA/MOTION/" + filename, file(motfile))

    @checkfilename
    def transferWav(self, wavfile):
        """Upload sound file (*.wav)"""
        dirname, filename = path.split(motfile)
        self._ftp.storbinary("STOR /MS/DATA/SOUND/" + filename, file(wavfile), 1024)

    @checkfilename
    def transferColor(self, colfile):
        """Upload color files (give path to *.col, *.tm will be uploaded as well)"""
        dirname, colfilename = path.split(colfile)
        tmfile = colfile.replace(".COL", ".TM")
        tmfilename = colfilename.replace(".COL", ".TM")
        self._ftp.storbinary("STOR /MS/CONFIG/" + tmfilename, file(tmfile), 1024)
        self._ftp.storlines("STOR /MS/CONFIG/" + colfilename, file(colfile))

    def viewWLANCONF(self):
        """Prints out the current settings in WLANCONF"""
        print self._ftp.retrlines("RETR /MS/open-r/system/conf/WLANCONF.TXT")

    def getWLANCONF(self):
        """Returns a dictionary with the current WLANCONF settings"""
        pass

    def setWLANCONF(self, wlanconfdict):
        """Sets the values in WLANCONF using a dictionary (use getWLANCONF)
        Example:
        >>> #rewrite old settings
        >>> wlanconf = dict(ESSID = "AIBO", WEPENABLE = 0, APMODE = 1, USE_DHCP = 1)
        >>> ftp.setWLANCONF(wlanconf)
        """
        pass

    def setCfgValue(self, name, value):
        """Sets a value in tekkotsu.cfg, if a list's name is used all entries will be overwritten"""
        pass

    def setCfgList(self, listname, value):    
        """Appends to a list (thresh, preload, etc.) in tekkotsu.cfg"""
        pass

    def delete(self, filename):
        """Delete a file"""
        self._ftp.delete(filename)

    def rename(self, fromname, toname):
        """Rename a file"""
        self._ftp.rename(fromname, toname)

    def mkd(self, dir):
        """Create directory, returns the full path name"""
        return self._ftp.mkd(dir)

    def getText(self, filename, outfile):
        """Retrives a text file and writes it to outfile"""
        self._ftp.retrlines("RETR " + filename, lambda s,w=outfile.write: w(s+"\n"))